function bigbattery
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Copyright Notice
% This software is the intellectual property of Brad Wyble and Patrick Craston, 2006
% It may be download and used freely.  
% This code may be included in other published works with the permission of the authors, 
% who can be contacted at bwyble @ gmail.com  (remove spaces)
 

%This script runs the model several times, simulating several conditions.
%Use flags in code following global var declaration to choose the
%conditions to be run.
%
%Conditions are:  Basic, T1+1 blank,  T2+1 blank
%8 lags between T1 and T2 are run (repeatedly) for each condition
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global T1BLANK T2BLANK T1BLANK2 genval genval2 BASICRSVP T2perf distval T1perf Swapencoding NUMLAYERS NUMSTREAMS
global BatT1Perf BatT2Perf BatT1CPerf BatT2CPerf Batswap T1batterydata T2batterydata Swapbatterydata
global Equalgens doubleencoding Batdoub doubledata  don0t dot1 dot2
global summate summateacc summatecount summate2 summateacc2 summatecount2 summateflag summateflag2
global SOA runlength onetarg singletrial

% allow T2+1 blank only for 2 target
if(onetarg)
    dot2 = 0;   %T2+1 blank
end

%model run stopwatch start
tic;

%set to 1 for equal T1 and T2
Equalgens = 0;

inputpatterns   %setup the RSVP streams
architecture    %setup the network

% specify lag - set to 0 to do 1:NUMSTREAMS lags - only lag 1 for single
% target
%initialize numlags var
numlags = 0;
if (onetarg)
    lag = 1;
    numlags = 1;
else
    lag = 0;
    numlags = NUMSTREAMS;
end

%variables for storing the raw activation values of individual trials
%summate(trialnum,timestep,layer,neuron)
%these variables are encoded in the function evaltargs.m
summate = zeros(400,runlength,4, 3);
summateacc = zeros(400, 3);
summatecount = 0;
summate2 = zeros(400,runlength,4, 3);
summateacc2 = zeros(400, 3);
summatecount2 = 0;

%check how many conditions were run - basic, blanks, etc
numconditions = 1;
if (dot1)
    numconditions = 2;
end
if(dot2)
    numconditions = 3;
end

% iterate the value of T1 and T2 over this range, at this resolution
resolution = .012;

varstart = floor(-.078/resolution);
varstop = floor(.078/resolution);

%run a single trial for test purpose
if(singletrial)
    varstart = 1;
    varstop = 1;
end

%So the strength of the target will iterate from -.x to +.x (added to the
%baseline value (as defined below)) in steps of .015
baseval = .526;


%this will happen conjunctively for every possible combination of T1 and T2

%So there are x+1+x = x steps for T1 and T2, leading to x runs of all 8 lags in the two target condition.
numTcombi = (-varstart+varstop+1)^2;
%for onetarget less possible target values
if(onetarg)
    numTcombi = (-varstart+varstop+1);
end

%behavioural storage variable
T1batterydata= 0; %values for T1 for each trial - encodes whatever is in neuron 1 so ok for single target
T2batterydata = 0; %values for T2 for each trial - neuron 2
Swapbatterydata = 0;
doubledata = 0;

offset = 1-varstart;

%Flags for storing output traces
summateflag2 = 0;
summateflag = 1;

%if onetarget do only one set of target values
T1start = varstart;
T1stop = varstop;

%stop running two nested loops for onetarget
if(onetarg)
    T2start = 0;
    T2stop = 0;
else
    T2start = varstart;
    T2stop = varstop;
end

%%%%%%%%%%%%%%%%%%  FIRST RUN: Basic Condition
if(don0t)
    %Flags used later to insert blanks into the stream  These variables are used in generateinput.m
    T1BLANK = 0;  %T1+1 blank
    T2BLANK = 0;   %T2+1 blank
    T1BLANK2 = 0;   %T1 +2 blank
    trialcounter = 0;   
    for(i=T1start:T1stop)
        for(j=T2start:T2stop)
            genval = baseval + i * resolution;
            genval2 = baseval + j *resolution;             
            first = i+offset; %i loop index
            if(singletrial | onetarg)
                second = 1;
            else
                second = j+offset; %j loop index
            end
            distval = 0.5;

            %print trial info
            sprintf('%d target combinations left in basic condition',numTcombi-trialcounter)
            trialcounter = trialcounter + 1;
            if (onetarg)
                sprintf('This set of lags: Target has value %0.4f - Distractors have value %0.4f',genval,distval)
            else
                sprintf('This set of lags: T1 has value %0.4f - T2 has value %0.2f - Distractors have value %0.4f',genval,genval2,distval)
            end


            %%%%%%this line actually runs the model%%%%%%
            runRSVP(BASICRSVP,lag)

            %these store the output
            T1batterydata(1,first,second,1:8) = reshape(T1perf,[1,1,1,8]);
            T2batterydata(1,first,second,1:8) = reshape(T2perf,[1,1,1,8]);
            Swapbatterydata(1,first,second,1:8) = reshape(Swapencoding,[1,1,1,8]);
            doubledata(1,first,second,1:8) =  reshape(doubleencoding,[1,1,1,8]);
        end
    end
end

%%%%%%%%%%%%%%%%%%SECOND RUN: T1+1 Blank Condition
if(dot1)
    T1BLANK2 = 0;
    T1BLANK = 1;
    T2BLANK = 0;
    summateflag = 0;
    summateflag2 = 1;
    trialcounter = 0;
    for(i=T1start:T1stop)
        for(j=T2start:T2stop)
            genval = baseval + i * resolution;
            genval2 = baseval + j* resolution;
            distval = 0.5;

            %print trial info
            sprintf('%d target combinations left in T1+1 blank condition',numTcombi-trialcounter)
            trialcounter = trialcounter + 1;
            if (onetarg)
                sprintf('This set of lags: Target has value %0.4f - Distractors have value %0.4f',genval,distval)
            else
                sprintf('This set of lags: T1 has value %0.4f - T2 has value %0.2f - Distractors have value %0.4f',genval,genval2,distval)
            end

            %%%%%%this line actually runs the model%%%%%%
            runRSVP(BASICRSVP,lag)

            first = i+offset;
            second = j+offset;

            T1batterydata(2,first,second,1:8) = reshape(T1perf,[1,1,1,8]);
            T2batterydata(2,first,second,1:8) = reshape(T2perf,[1,1,1,8]);
            Swapbatterydata(2,first,second,1:8) = reshape(Swapencoding,[1,1,1,8]);
            doubledata(2,first,second,1:8) =  reshape(doubleencoding,[1,1,1,8]);

        end
    end
end

%%%%%%%%%%%%%%%%%%%%THIRD RUN: T2+1 blank condition
if(dot2)
    summateflag2 = 0;
    T1BLANK = 0;
    T2BLANK = 1;
    trialcounter = 0;
    for(i=T1start:T1stop)
        for(j=T2start:T2stop)
            genval = baseval + i * resolution;
            genval2 = baseval + j * resolution;
            distval = 0.5;

            first = i+offset;
            second = j+offset;

            %print trial info
            sprintf('%d target combinations left in T2+1 blank condition',numTcombi-trialcounter)
            trialcounter = trialcounter + 1;
            if (onetarg)
                sprintf('This set of lags: Target has value %0.4f - Distractors have value %0.4f',genval,distval)
            else
                sprintf('This set of lags: T1 has value %0.4f - T2 has value %0.2f - Distractors have value %0.4f',genval,genval2,distval)
            end


            %%%%%this line actually runs the model%%%%%%
            runRSVP(BASICRSVP,lag)

            T1batterydata(3,first,second,1:8) = reshape(T1perf,[1,1,1,8]);
            T2batterydata(3,first,second,1:8) = reshape(T2perf,[1,1,1,8]);
            Swapbatterydata(3,first,second,1:8) = reshape(Swapencoding,[1,1,1,8]);
            doubledata(3,first,second,1:8) =  reshape(doubleencoding,[1,1,1,8]);
        end
    end
end


%go through all of the stored output of the model and compute the accuracies.

%onetarget only has T1 data
BatT1Perf = zeros(numconditions,numlags);

if(~onetarg)
    BatT2Perf = zeros(numconditions,numlags);
    BatT2CPerf = zeros(numconditions,numlags);
    BatT1CPerf = zeros(numconditions,numlags);
    Batswap = zeros(numconditions,numlags);
    Batdoub = zeros(numconditions,numlags);
end

% copy target performance data to battery wide variable
for(t = 1:numconditions)
    for(i = 1:size(T1batterydata,2))%first (i) loop index
        for(j = 1:size(T1batterydata,3))%second (j) loop index
            for(k = 1:numlags)
                if(~onetarg)
                    doub = doubledata(t,i,j,k);
                    t2 = T2batterydata(t,i,j,k);
                end
                t1 = T1batterydata(t,i,j,k);
                if(t1) %not zero
                    BatT1Perf(t,k)= BatT1Perf(t,k)+ 1;
                end
                if(~onetarg)
                    if(t2)
                        BatT2Perf(t,k)= BatT2Perf(t,k)+ 1;
                    end
                    Batdoub(t,k) = Batdoub(t,k) + doub;
                    if(t1 & t2)
                        BatT2CPerf(t,k)= BatT2CPerf(t,k)+ 1;
                    end
                    if(t2)
                        BatT1CPerf(t,k)= BatT1CPerf(t,k)+ 1;
                    end
                    if(t1 & t2)
                        Batswap(t,k) = Batswap(t,k) + Swapbatterydata(t,i,j,k);
                    end
                end
            end
        end
    end
    for(k = 1:numlags)
        if(~onetarg)
            BatT2Perf(t,k) = BatT2Perf(t,k) / numTcombi;
            BatT1CPerf(t,k) = BatT2CPerf(t,k) / BatT2Perf(t,k); 
            BatT2CPerf(t,k) = BatT2CPerf(t,k) / BatT1Perf(t,k);  
            Batdoub(t,k) = Batdoub(t,k)/BatT2CPerf(t,k);
            Batswap(t,k) = Batswap(t,k)/BatT2CPerf(t,k);         
        end        
        BatT1Perf(t,k) = BatT1Perf(t,k) / numTcombi;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Save the behavioural output
%Duration
duration = floor(toc/60);
sprintf('Model ran for %d minutes',duration)

%accuracy and model performance on target detection
if(onetarg)
    outfile = [ 'STSToutput_1targ_' num2str(SOA) 'ms' ];
    save(outfile, 'duration', 'summate','summatecount','summateacc', 'summate2','summatecount2','summateacc2','BatT1Perf','T1batterydata');
else
    outfile = [ 'STSToutput_' num2str(SOA) 'ms' ];
    save(outfile, 'duration', 'summate','summatecount','summateacc', 'summate2','summatecount2','summateacc2','BatT2Perf','BatT1Perf','BatT2CPerf','BatT1CPerf','Batswap','Batdoub', 'doubledata','T1batterydata','T2batterydata','Swapbatterydata' );
end
